package org.implab.gradle.containers;

import org.gradle.api.GradleException;
import org.gradle.api.Plugin;
import org.gradle.api.Project;
import org.gradle.api.artifacts.Dependency;
import org.gradle.api.file.ProjectLayout;
import org.gradle.api.tasks.Copy;
import org.gradle.api.tasks.Delete;
import org.gradle.api.tasks.TaskProvider;
import org.implab.gradle.containers.cli.ImageName;
import org.implab.gradle.containers.tasks.BuildImage;
import org.implab.gradle.containers.tasks.PushImage;
import org.implab.gradle.containers.tasks.SaveImage;

public class ContainerPlugin implements Plugin<Project> {

    public static final String BUILD_GROUP = "build";

    public static final String ARCHIVE_CONFIGURATION = "archive";

    public void apply(Project project) {
        ProjectLayout layout = project.getLayout();

        project.getPluginManager().apply(ContainerBasePlugin.class);

        var basePlugin = project.getPlugins().findPlugin(ContainerBasePlugin.class);
        if (basePlugin == null)
            throw new GradleException("The container-base plugin fails to be applied");

        var containerExtension = basePlugin.getContainerExtension();        

        project.getConfigurations().create(Dependency.DEFAULT_CONFIGURATION, c -> {
            c.setCanBeConsumed(true);
            c.setCanBeResolved(false);
        });

        project.getConfigurations().create(ARCHIVE_CONFIGURATION, c -> {
            c.setCanBeConsumed(true);
            c.setCanBeResolved(false);
        });

        TaskProvider<Copy> processResourcesTask = project.getTasks().register("processResources", Copy.class, t -> {
            t.setGroup(BUILD_GROUP);
            t.from(layout.getProjectDirectory().dir("src/main"));
            t.into(containerExtension.getContextDirectory());
        });

        TaskProvider<BuildImage> buildImageTask = project.getTasks().register("buildImage", BuildImage.class, t -> {
            t.setGroup(BUILD_GROUP);
            t.dependsOn(processResourcesTask);

            t.getContextDirectory().set(containerExtension.getContextDirectory());
            t.getImageIdFile().set(project.getLayout().getBuildDirectory().file("iid.json"));

            t.getImageName().set(containerExtension.getImageName().map(ImageName::toString));
        });

        project.getTasks().register("clean", Delete.class, t -> {
            t.delete(layout.getBuildDirectory());
        });

        project.getTasks().register("build", t -> {
            t.setGroup(BUILD_GROUP);
            t.dependsOn(buildImageTask);
        });

        project.getTasks().register("pushImage", PushImage.class, t -> {
            t.dependsOn(buildImageTask);
            t.getImageName().set(buildImageTask.flatMap(BuildImage::getImageName));
        });

        project.getTasks().register("saveImage", SaveImage.class, t -> {
            t.dependsOn(buildImageTask);
            t.getExportImages().add(buildImageTask.flatMap(BuildImage::getImageName));
        });

        project.getArtifacts().add(Dependency.DEFAULT_CONFIGURATION, buildImageTask);
    }
}
